<?php

namespace Paysera\Magento2Paysera\Controller\Index;

use Magento\Framework\App\Action\Action;
use Magento\Framework\App\Action\Context;
use Magento\Framework\App\ResponseInterface;
use Magento\Framework\Controller\ResultInterface;
use Magento\Store\Model\ScopeInterface;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Sales\Api\OrderRepositoryInterface;
use Magento\Framework\Exception\LocalizedException;
use Paysera\Magento2Paysera\Helper\Data as DataHelper;
use Paysera\Magento2Paysera\Model\PayseraConfigProvider;
use WebToPay;
use Exception;
use Magento\Framework\Controller\Result\Raw;
use Magento\Framework\Api\SearchCriteriaBuilder;

class Callback extends Action
{
    protected ScopeConfigInterface $scopeConfig;
    protected OrderRepositoryInterface $orderRepository;
    protected Raw $rawResult;
    protected SearchCriteriaBuilder $searchCriteriaBuilder;
    protected DataHelper $dataHelper;

    public function __construct(
        Context $context,
        ScopeConfigInterface $scopeConfig,
        OrderRepositoryInterface $orderRepository,
        Raw $rawResult,
        SearchCriteriaBuilder $searchCriteriaBuilder,
        DataHelper $dataHelper
    ) {
        $this->scopeConfig = $scopeConfig;
        $this->orderRepository = $orderRepository;
        $this->rawResult = $rawResult;
        $this->searchCriteriaBuilder = $searchCriteriaBuilder;
        $this->dataHelper = $dataHelper;

        parent::__construct($context);
    }

    /**
     * @return ResponseInterface|ResultInterface|void
     *
     * @throws Exception
     */
    public function execute()
    {
        $paysera_config = $this->scopeConfig->getValue(
            PayseraConfigProvider::PAYSERA_PAYMENT,
            ScopeInterface::SCOPE_STORE
        );

        $requestData = $this->dataHelper->getRequestData();

        try {
            $response = WebToPay::validateAndParseData(
                $requestData,
                $paysera_config['projectid'],
                $paysera_config['sign_password']
            );

            if ((int) $response['status'] === PayseraConfigProvider::RESPONSE_STATUS_CONFIRMED) {
                $searchCriteria = $this->searchCriteriaBuilder
                    ->addFilter('increment_id', $response['orderid'], 'eq')
                    ->create()
                ;

                $orders = $this->orderRepository->getList($searchCriteria)->getItems();

                if (count($orders) === 0) {
                    throw new Exception('Order not found.');
                }

                $order = array_values($orders)[0];
                $orderTotalCents = $order->getGrandTotal() * 100;

                $money = [
                    'amount' => (string)$orderTotalCents,
                    'currency' => $order->getOrderCurrencyCode()
                ];

                $isPaymentCorrect = $this->checkPayment($money, $response);

                if ($isPaymentCorrect) {
                    $order->setStatus(
                        $paysera_config['paysera_order_status']['order_status']
                    )->save();

                    return $this->rawResult->setContents('OK');
                }
            }
        } catch (Exception $e) {
            return $this->rawResult->setContents(get_class($e) . ': ' . $e->getMessage());
        }
    }

    /**
     * @param array $orderMoney
     * @param array $response
     *
     * @return bool
     *
     * @throws LocalizedException
     */
    public function checkPayment($orderMoney, $response)
    {
        $checkConvert = array_key_exists('payamount', $response);
        $orderAmount = $orderMoney['amount'];
        $orderCurrency = $orderMoney['currency'];

        if ($response['amount'] !== $orderAmount || ($checkConvert && $response['payamount'] !== $orderAmount)) {
            throw new LocalizedException(__('Amounts do not match'));
        }

        if ($response['currency'] !== $orderCurrency
            || ($checkConvert && $response['paycurrency'] !== $orderCurrency)
        ) {
            throw new LocalizedException(__('Currencies do not match'));
        }

        return true;
    }
}
